// file      : xsd-frontend/semantic-graph/schema.hxx
// author    : Boris Kolpackov <boris@codesynthesis.com>
// copyright : Copyright (c) 2005-2006 Code Synthesis Tools CC
// license   : GNU GPL v2 + exceptions; see accompanying LICENSE file

#ifndef XSD_FRONTEND_SEMANTIC_GRAPH_SCHEMA_HXX
#define XSD_FRONTEND_SEMANTIC_GRAPH_SCHEMA_HXX

#include <xsd-frontend/semantic-graph/elements.hxx>
#include <xsd-frontend/semantic-graph/namespace.hxx>

namespace XSDFrontend
{
  namespace SemanticGraph
  {
    //
    //
    class Schema;


    //
    //
    class Implies: public virtual Contains
    {
    public:
      Path
      file () const
      {
        return file_;
      }

    protected:
      friend class Bits::Graph<Node, Edge>;

      Implies (Path const& file)
          : file_ (file)
      {
      }

    private:
      Path file_;
    };


    //
    //
    class Sources: public virtual Contains
    {
    public:
      Path
      file () const
      {
        return file_;
      }

    protected:
      friend class Bits::Graph<Node, Edge>;

      Sources (Path const& file)
          : file_ (file)
      {
      }

    private:
      Path file_;
    };


    //
    //
    class Includes: public virtual Contains
    {
    public:
      Path
      file () const
      {
        return file_;
      }

    protected:
      friend class Bits::Graph<Node, Edge>;

      Includes (Path const& file)
          : file_ (file)
      {
      }

    private:
      Path file_;
    };


    //
    //
    class Imports: public virtual Contains
    {
    public:
      Path
      file () const
      {
        return file_;
      }

    protected:
      friend class Bits::Graph<Node, Edge>;

      Imports (Path const& file)
          : file_ (file)
      {
      }

    private:
      Path file_;
    };


    //
    //
    class Schema: public virtual Container,
                  public virtual Scope,
                  private Bits::Graph<Node, Edge>,
                  public NonCopyable
    {
      typedef
      Cult::Containers::Vector<Contains*>
      ContainedList;

    public:
      Schema (Path const& file, UnsignedLong line, UnsignedLong column)
          : SemanticGraph::Node (file, line, column), contained_ (0)
      {
      }

    public:
      Boolean
      contained () const
      {
        return contained_begin () != contained_end ();
      }

      typedef
      Bits::PointerIterator<ContainedList::ConstIterator>
      ContainedIterator;

      ContainedIterator
      contained_begin () const
      {
        return contained_.begin ();
      }

      ContainedIterator
      contained_end () const
      {
        return contained_.end ();
      }

      virtual NamesIteratorPair
      find (Name const& name) const;

    private:
      Bits::Graph<SemanticGraph::Node, Edge>&
      graph ()
      {
        return *this;
      }

    public:
      using Bits::Graph<SemanticGraph::Node, Edge>::new_edge;

      template <typename T>
      T&
      new_node (Path const& file, UnsignedLong line, UnsignedLong column)
      {
        return graph ().new_node<T> (file, line, column);
      }

      template <typename T, typename A0>
      T&
      new_node (Path const& file, UnsignedLong line, UnsignedLong column,
                A0 const& a0)
      {
        return graph ().new_node<T> (file, line, column, a0);
      }

      template <typename T, typename A0, typename A1>
      T&
      new_node (Path const& file, UnsignedLong line, UnsignedLong column,
                A0 const& a0, A1 const& a1)
      {
        return graph ().new_node<T> (file, line, column, a0, a1);
      }

      template <typename T, typename A0, typename A1, typename A2>
      T&
      new_node (Path const& file, UnsignedLong line, UnsignedLong column,
                A0 const& a0, A1 const& a1, A2 const& a2)
      {
        return graph ().new_node<T> (file, line, column, a0, a1, a2);
      }

      template <typename T, typename A0, typename A1, typename A2,
                typename A3>
      T&
      new_node (Path const& file, UnsignedLong line, UnsignedLong column,
                A0 const& a0, A1 const& a1, A2 const& a2, A3 const& a3)
      {
        return graph ().new_node<T> (file, line, column, a0, a1, a2, a3);
      }

    protected:
      //@@ gcc bug #21146
      //
      friend class Bits::Graph<SemanticGraph::Node, Edge>;

      using Scope::add_edge_left;
      using Container::add_edge_left;

      using Node::add_edge_right;

      Void
      add_edge_right (Contains& c)
      {
        contained_.push_back (&c);
      }

    private:
      ContainedList contained_;

      mutable NamesList names_;
    };
  }
}

#endif  // XSD_FRONTEND_SEMANTIC_GRAPH_SCHEMA_HXX
